/***************************************************
* By CCSC compiler
*  File:   20240826_K733_DSPradio_18346.c
*  Description: DSP radio with K0913
*  RA0 in switch_aux
*  RA1 in na
*  RA2 out pwm
*  RA3 in reset
*  RA4 in switch_set
*  RA5 out decoder_enable
*  RB4 out led_fm
*  RB5 out led_am
*  RB6 out SDA
*  RB7 out SCL
*  RC0  out decoder0 
*  RC1 out decoder1
*  RC2 out decoder2
*  RC3 out power
*  RC4 in switch_up
*  RC5 in switch_down
*  RC6 in switch_fm
*  RC7 out lcd_power
*  COPYRIGHT 2024 MYCOMKITS.COM, owned by CNET LIMITED
   ***************************************************/
#include <16f18346.h>
//#device ADC=10
#include <stdlib.h>
#include <math.h>
#fuses RSTOSC_HFINTRC_PLL
#fuses NOCLKOUT,CKS,NOFCMEN,NOMCLR
#fuses PUT,NOWDT
#fuses BROWNOUT
#fuses NOPPS1WAY,NOSTVREN,NODEBUG,NOWRT,NOLVP
#fuses NOPROTECT,NOCPD
#use   delay(clock = 4000000)

#pin_select PWM5=PIN_A2
#use pwm(PWM5, FREQUENCY=1k)
#pin_select SCL1OUT=PIN_B7
#pin_select SDA1IN=PIN_B6
#use I2C(Master, SLOW, I2C1)

// Input/Output Pin
#define Amode      0xDB    // All Input, except A2, 5
#define AmodePU      0x11   // pullup A0,4
#define Bmode      0x00    // All output
#define BmodePU      0x00  // 
#define Cmode      0x70   // all output, except C4,5
#define CmodePU      0x70   // pullups C4,5,6

#define PWM_FREQUENCY 1000  //1KHZ

#define up_sw PIN_A4
#define down_sw PIN_C5
#define set_sw PIN_C4
#define aux_sw PIN_C6
#define mode_sw PIN_A0
//#define aux_sw PIN_A0
//#define mode_sw PIN_C6
#define power_level PIN_A1
#define P1A PIN_A2
#define LED_FM PIN_B4
#define LED_AM PIN_B5
#define DecoderEnb PIN_A5
#define Decoder0 PIN_C0
#define Decoder1 PIN_C1
#define Decoder2 PIN_C2
#define LCD_LIGHT PIN_C7
#define PowerEnable PIN_C3

// Variables
int start_flag=0;
int mode=0; //0=top, 1=p, 2=i, 3=d, 4=Bconst
int freq_mode=0;    //0=FM, 1=AM
long freq_fm=795;
long freq_am=945;
float Freq;
long default_fm=795;
long default_am=945;
int CONTRAST=0x30;
int Index=0;
signed int Index_fm=0;
signed int Index_am=0;
long SetFreq;
signed long RSSI;
unsigned long CH;
int sw_flag=0;

//$B!!Dj5A(B
struct FMKyoku {
   float freq;
   char name[17];
};

struct AMKyoku {
   long freq;
   char name[17];
};
//$B!!@k8@(B
struct FMKyoku FMstation[7] = {
   {76.7, "Flower Radio    "},
   {79.5, "NACK5           "},
   {81.3, "J-WAVE          "},
   {82.5, "NHK FM Tokyo    "},
   {90.5, "TBS Radio       "},
   {91.6, "Bunka Hoso      "},
   {93.0, "Nippon Hoso     "}
};

struct AMKyoku AMstation[7] = {
   {594, "NHK Tokyo1      "},
   {693, "NHK Tokyo2      "},
   {810, "AFN             "},
   {927, "NHK Yamanashi1  "},
   {954, "TBS Radio       "},
   {1134, "Bunka Hoso      "},
   {1242, "Nippon Hoso     "}
};

// Function
extern void Initialize(void);
extern void CheckSwitch();
extern void CheckStart();
//extern void SetMode();  //FM or AM
extern void Test();
extern void Test2();
extern void Test3();
extern void PowerCheck();
extern void RadioInit();
extern void Disp();
extern void DSP_Init();
extern void MagicEye();
extern void TunigBar();
extern void Ledbar(int x);
extern void lcd_data(unsigned char x);
extern void lcd_cmd(unsigned char x);

// DSP register setting
void SetReg(int8 reg, int16 data)
{
    //int16 temp;
    int8 temp2, temp3;
    //temp=data;
    
    temp2=data >> 8;
    temp3=data & 0xFF;

    // I2C$B=q$-9~$_A`:n(B
    i2c_start();
    //i2c_write(0x35);         // $B%9%l!<%V%"%I%l%9(B
    i2c_write(0x6A);         // $B%9%l!<%V%"%I%l%9(B, left shifted 0x35
    i2c_write(reg);          // $B%l%8%9%?%"%I%l%9(B
    i2c_write(temp2);    // $B>e0L%P%$%H(B
    i2c_write(temp3);  // $B2<0L%P%$%H(B
    i2c_stop();
}

void DSP_Init(void)
{
    Index_fm = read_eeprom(0); //$BJ|Aw6IHV9f(B
    Index_am = read_eeprom(1); //$BJ|Aw6IHV9f(B
    freq_mode = read_eeprom(2); //$BJ|Aw6IHV9f(B

    if(freq_mode==0)    //0=FM, 1=AM
    {
        SetFreq = (float)((FMstation[Index_fm].freq)/0.05);
        SetReg(0x16, 0x0002);   //FM, 32.768k
        SetReg(0x03, (SetFreq & 0x0FFF) | 0x8000);
        output_low(LED_FM);
        output_high(LED_AM);
    }
    else
    {
        SetFreq = (long)(AMstation[Index_am].freq);
        SetReg(0x16, 0x8002);   //AM, 32.768k
        SetReg(0x17, SetFreq & 0x07FF);
        output_high(LED_FM);
        output_low(LED_AM);
    }

    SetReg(0x1D, 0x0000);   //02 ch mode or 00 register setting mode
    SetReg(0x04, 0xE180);
    SetReg(0x0A, 0x0000);
    SetReg(0x05, 0x1800);
    SetReg(0x0F, 0x881F);   //volume 0dBm
    //Disp();
}

void Get_freq(void)
{
    if(freq_mode==0)    //$B$U$)!)(BFM
    {
        unsigned int8 high_byte, low_byte;
        i2c_start();
        i2c_write(0x6A);           // $B%9%l!<%V%"%I%l%9!J=q$-9~$_%b!<%I!K(B
        i2c_write(0x03);           // $B%l%8%9%?%"%I%l%9(B for FM
        i2c_start();               // $B:F%9%?!<%H(B
        i2c_write(0x6B);    // $B%9%l!<%V%"%I%l%9!JFI$_9~$_%b!<%I!K(B
        high_byte = i2c_read();   // $B>e0L%P%$%H$rFI$_<h$j!"(BACK$B$rJV$9(B
        low_byte = i2c_read();    // $B2<0L%P%$%H$rFI$_<h$j!"(BNACK$B$rJV$9(B
        i2c_stop();
        CH=high_byte;
        CH = (CH << 8) | low_byte;  // $B>e0L%P%$%H$H2<0L%P%$%H$r7k9g(B
        Freq=CH & 0x0FFF;
        CH = CH & 0x07FF;
        Freq=Freq * 0.05;
    }
    else    // For AM
    {
        /*
        unsigned int8 high_byte, low_byte;
        i2c_start();
        i2c_write(0x6A);           // $B%9%l!<%V%"%I%l%9!J=q$-9~$_%b!<%I!K(B
        i2c_write(0x17);           // $B%l%8%9%?%"%I%l%9(B for AM
        i2c_start();               // $B:F%9%?!<%H(B
        i2c_write(0x6B);    // $B%9%l!<%V%"%I%l%9!JFI$_9~$_%b!<%I!K(B
        high_byte = i2c_read();   // $B>e0L%P%$%H$rFI$_<h$j!"(BACK$B$rJV$9(B
        low_byte = i2c_read();    // $B2<0L%P%$%H$rFI$_<h$j!"(BNACK$B$rJV$9(B
        i2c_stop();
        CH=high_byte;
        CH = (CH << 8) | low_byte;  // $B>e0L%P%$%H$H2<0L%P%$%H$r7k9g(B
        Freq=CH & 0x0FFF;
        CH = CH & 0x07FF;
        */
        CH = (long)(AMstation[Index_am].freq);
        CH = CH & 0x07FF;
    }
}

void Get_rssi(void)
{
    if(freq_mode==0)    //FM
    {
        unsigned int16 temp;
        unsigned int8 high_byte, low_byte;

        i2c_start();
        i2c_write(0x6A);            // $B%9%l!<%V%"%I%l%9!J=q$-9~$_%b!<%I!K(B
        i2c_write(0x12);            // $B%l%8%9%?%"%I%l%9(B
        i2c_start();                // $B:F%9%?!<%H(B
        i2c_write(0x6B);     // $B%9%l!<%V%"%I%l%9!JFI$_9~$_%b!<%I!K(B
        high_byte = i2c_read(1);    // $B>e0L%P%$%H$rFI$_<h$j!"(BACK$B$rJV$9(B
        low_byte = i2c_read(0);     // $B2<0L%P%$%H$rFI$_<h$j!"(BNACK$B$rJV$9(B
        i2c_stop();
        temp = (high_byte << 8) | low_byte;  // $B>e0L%P%$%H$H2<0L%P%$%H$r7k9g(B
        RSSI = (temp & 0x00F8) >> 3;         // $BI,MW$J(B5$B%S%C%H$rCj=P$7!"1&%7%U%H(B
        RSSI = (RSSI * 3) - 100;    //x3 minus 100 dBm
    }
    else
    {
        unsigned int16 temp;
        unsigned int8 high_byte, low_byte;

        i2c_start();
        i2c_write(0x6A);            // $B%9%l!<%V%"%I%l%9!J=q$-9~$_%b!<%I!K(B
        i2c_write(0x24);            // $B%l%8%9%?%"%I%l%9(B
        i2c_start();                // $B:F%9%?!<%H(B
        i2c_write(0x6B);     // $B%9%l!<%V%"%I%l%9!JFI$_9~$_%b!<%I!K(B
        high_byte = i2c_read(1);    // $B>e0L%P%$%H$rFI$_<h$j!"(BACK$B$rJV$9(B
        low_byte = i2c_read(0);     // $B2<0L%P%$%H$rFI$_<h$j!"(BNACK$B$rJV$9(B
        i2c_stop();
        //temp = (high_byte << 8) | low_byte;  // $B>e0L%P%$%H$H2<0L%P%$%H$r7k9g(B
        RSSI = high_byte & 0x1F;         // 5$B%S%C%H$rCj=P(B
        RSSI = (RSSI * 3) - 100;    //x3 minus 100 dBm
    }
}

void Disp(void)
{
    if(freq_mode==0)
    {
        lcd_cmd(0x80);
        printf(lcd_data,"%S", FMstation[Index_fm].name);
        lcd_cmd(0xC0);
        printf(lcd_data,"%2.1fMHz", FMstation[Index_fm].freq);
    }
    else
    {
        lcd_cmd(0x80);
        printf(lcd_data,"%S", AMstation[Index_am].name);
        lcd_cmd(0xC0);
        printf(lcd_data,"%4luKHz         ", AMstation[Index_am].freq);
    }

    //$B<u?.<~GH?t%j!<%I(B
    Get_freq();
    //$B<u?.%l%Y%k!!(BdBm$BI=<((B
    //if(freq_mode==0) 
    //{
        Get_rssi();
        //lcd_cmd(0xC0);
        //printf(lcd_data,"%2.1fMHz  %3lddBm ", Freq, RSSI);
        lcd_cmd(0xC8);
        printf(lcd_data,"%3lddBm  ", RSSI);
        //else printf(lcd_data,"%4luKHz         ", CH & 0x03FF);
        //else printf(lcd_data,"%4luKHz         ", AMstation[Index_am].freq);
    //}
}

void lcd_data(unsigned char data)
{
	i2c_start();
	i2c_write(0x7C); // OLED slave address
	i2c_write(0x40); // Control byte Co = 0, D / C# = 0
	i2c_write(data); // Set charge pump
	i2c_stop();
}

void lcd_cmd(unsigned char cmd)
{
	i2c_start();
	i2c_write(0x7C); // OLED slave address
	i2c_write(0x00); // Control byte Co = 0, D / C# = 0
	i2c_write(cmd); // Set charge pump
	i2c_stop();
   
   if((cmd==0x01) || (cmd==0x02)) delay_ms(2);
   else delay_us(30);
}

void lcd_init(void)
{
    //int CONTRAST=0;
	delay_ms(100);
	lcd_cmd(0x38);
	lcd_cmd(0x39);
	lcd_cmd(0x14);
	
	lcd_cmd(0x70 + (CONTRAST & 0x0F));
	lcd_cmd(0x5C + (CONTRAST >> 4));
	lcd_cmd(0x6B);  //for 3.3V
	delay_ms(300);
	//lcd_cmd(0x38);
	lcd_cmd(0x0C);
	lcd_cmd(0x01);
}

void lcd_clear(void)
{
	lcd_cmd(0x01);
}

//write_LONG_to_eeprom;
void WRITE_LONG_EEPROM(int addr, long data) 
{
   INT8 i;
   FOR(i = 0; i < 2; i++)
   {
      write_eeprom(addr + i, *((INT8 *)&data + i));
      delay_ms(20);
   }
}

//read_LONG_from_eeprom;
long READ_LONG_EEPROM(int addr) 
{
   
   INT8 i;
   LONG data = 0;
   FOR(i = 0; i < 2; i++)
   {
      *((INT8*)&data + i) = read_eeprom(addr +  i);
      delay_ms(20);
   }

   RETURN(data);
}

// Main Routine
void main() 
{
    Initialize();
    lcd_init();
    DSP_Init();
    Disp();
    output_low(LCD_LIGHT);
    // Main routine
    while(1)
    {   
        
        PowerCheck();
        CheckSwitch();
        Disp();
        MagicEye();
        TunigBar();
    }
}

// Initialize
void Initialize()
{
    //I / O Port Setup
    set_tris_a(Amode);
    port_a_pullups(AmodePU);
    set_tris_b(Bmode);
    port_b_pullups(BmodePU);
    set_tris_c(Cmode);
    port_c_pullups(CmodePU);
    output_low(P1A);

    output_float(PIN_B7);
    output_float(PIN_B6);

    setup_timer_2(T2_DIV_BY_16, 249, 1); //8us
    set_timer2(0x0);   // timer 1 setting almost 0.5sec
    setup_pwm5(PWM_ENABLED);
    set_pwm5_duty(0);

    output_high(PowerEnable);
    start_flag=1;
    output_high(DecoderEnb);

    Index_fm = read_eeprom(0);
    if((Index_fm > 6) || (Index_fm <0))
    {
       Index_fm = 1;    
       write_eeprom(0, Index_fm);
    }

    Index_am = read_eeprom(1);
    if((Index_am > 6) || (Index_am <0))
    {
       Index_am = 0;    
       write_eeprom(1, Index_am);
    }

    freq_mode = read_eeprom(2);
    if(freq_mode > 1)    //0=FM, 1=AM
    {
       freq_mode = 0;    
       write_eeprom(2, freq_mode);
    }
    if(freq_mode==1) 
    {
        output_low(LED_AM); //active low
        output_high(LED_FM); //active low
    }
    else 
    {
        output_low(LED_FM); //active low
        output_high(LED_AM); //active low
    }
   
   freq_fm = READ_LONG_EEPROM(5);
   if(freq_fm < 700 || freq_fm > 1100)
   {
        freq_fm = default_fm;
        WRITE_LONG_EEPROM(5, default_fm);
   }

   freq_am = READ_LONG_EEPROM(7);
   if(freq_am < 500 || freq_fm > 1100)
   {
        freq_am = default_am;
        WRITE_LONG_EEPROM(7, default_am);
   }

   delay_ms(100);
}

void CheckSwitch()
{
    if(input(mode_sw)==0)
    {
        delay_ms(20);
        if(input(mode_sw) == 0)
        {
            if(freq_mode==0) 
            {
                freq_mode=1;   //1=AM, 0=FM
                Index_am = read_eeprom(1); //$BJ|Aw6IHV9f(B
                output_low(LED_AM); //active low
                output_high(LED_FM); //active low
                SetFreq = (long)(AMstation[Index_am].freq);
                SetReg(0x16, 0x8002);   //AM, 32.768k
                delay_ms(1);
                SetReg(0x17, SetFreq & 0x07FF);
                delay_ms(1);
                
            }
            else 
            {
                freq_mode=0;
                Index_fm = read_eeprom(0); //$BJ|Aw6IHV9f(B
                output_low(LED_FM); //active low
                output_high(LED_AM); //active low
                SetFreq = (float)((FMstation[Index_fm].freq)/0.05);
                SetReg(0x16, 0x0002);   //FM, 32.768k
                delay_ms(1);
                SetReg(0x03, (SetFreq & 0x0FFF) | 0x8000);
                delay_ms(1);
                
            }
            sw_flag = 1;
        }
        while(input(mode_sw) == 0) {}
        delay_ms(20);
    }
    //
    if(input(up_sw) == 0)
    {
        delay_ms(20);
        if(input(up_sw) == 0)
        {
           if(freq_mode == 0)
            {
                Index_fm=Index_fm + 1;
                if(Index_fm>6) Index_fm=6;
                SetFreq = (float)((FMstation[Index_fm].freq)/0.05);
                SetReg(0x03, (SetFreq & 0x0FFF) | 0x8000);
                //SetReg(0x03, SetFreq | 0x8000);
                sw_flag = 1;
            }
            else
            {
                Index_am=Index_am + 1;
                if(Index_am>6) Index_am=6;
                SetFreq = (long)(AMstation[Index_am].freq);
                SetReg(0x17, SetFreq);
                SetReg(0x17, (SetFreq & 0x07FF));
                sw_flag = 1;
            }
        }
        while(input(up_sw) == 0) {}
        delay_ms(20);
    }
    
    if(input(down_sw) == 0)
    {
        delay_ms(20);
        if(input(down_sw) == 0)
        {
           if(freq_mode == 0)
            {
               Index_fm=Index_fm - 1;
               if(Index_fm<0) Index_fm=0;
               SetFreq = (float)((FMstation[Index_fm].freq)/0.05);
                SetReg(0x03, (SetFreq & 0x0FFF) | 0x8000);
                //SetReg(0x03, SetFreq | 0x8000);
                 sw_flag = 1;
            }
            else
            {
                Index_am=Index_am - 1;
                if(Index_am<0) Index_am=0;
                SetFreq = (long)(AMstation[Index_am].freq);
                //SetReg(0x17, SetFreq);
                SetReg(0x17, (SetFreq & 0x07FF));
                sw_flag = 1;
            }
        }
        while(input(down_sw) == 0) {}
        delay_ms(20);
    }
//
    if(sw_flag == 1)
    {
        write_eeprom(0, Index_fm);
        write_eeprom(1, Index_am);
        write_eeprom(2, freq_mode);
        sw_flag = 0;
    }
   
}

void PowerCheck()
{
    output_high(PowerEnable);   //for test
    if(input(power_level)==1)
    {
        delay_ms(20);
        if(input(power_level)==1)
        {
            if(start_flag==0) 
            {
                output_high(PowerEnable);
                start_flag=1;
                while(input(power_level)==1) {}
                delay_ms(20);
            }
            else output_low(PowerEnable);
        }
    }
}

void MagicEye()
{

        if(RSSI > -76) set_pwm5_duty(1000);
        else if(RSSI > -78) set_pwm5_duty(800);
        else if(RSSI > -80) set_pwm5_duty(600);
        else if(RSSI > -82) set_pwm5_duty(400);
        else if(RSSI > -84) set_pwm5_duty(200);
        else if(RSSI > -86) set_pwm5_duty(100);        
        else set_pwm5_duty(50);

}

void TunigBar()
{
    if(freq_mode==0)    //for FM
    {
        if(Freq < 78) Ledbar(0);
        else if(Freq < 80) Ledbar(1);
        else if(Freq < 83) Ledbar(2);
        else if(Freq < 86) Ledbar(3);
        else if(Freq < 89) Ledbar(4);
        else if(Freq < 92) Ledbar(5);
        else if(Freq < 94) Ledbar(6);
        else Ledbar(7);
    }
    else
    {
        if(CH < 600) Ledbar(0);
        else if(CH < 700) Ledbar(1);
        else if(CH < 800) Ledbar(2);
        else if(CH < 900) Ledbar(3);
        else if(CH < 1000) Ledbar(4);
        else if(CH < 1100) Ledbar(5);
        else if(CH < 1200) Ledbar(6);
        else Ledbar(7);
    }
}

void Ledbar(int dat)
{
    if(dat == 0)    //0=low freq, 7=high freq
    {
        output_high(Decoder0);
        output_high(Decoder1);
        output_high(Decoder2);
    }
    else if(dat == 1)    //0=low freq, 7=high freq
    {
        output_low(Decoder0);
        output_high(Decoder1);
        output_high(Decoder2);
    }
    else if(dat == 2)    //0=low freq, 7=high freq
    {
        output_high(Decoder0);
        output_low(Decoder1);
        output_high(Decoder2);
    }
    else if(dat == 3)    //0=low freq, 7=high freq
    {
        output_low(Decoder0);
        output_low(Decoder1);
        output_high(Decoder2);
    }
    else if(dat == 4)    //0=low freq, 7=high freq
    {
        output_high(Decoder0);
        output_high(Decoder1);
        output_low(Decoder2);
    }
    else if(dat == 5)    //0=low freq, 7=high freq
    {
        output_low(Decoder0);
        output_high(Decoder1);
        output_low(Decoder2);
    }
    else if(dat == 6)    //0=low freq, 7=high freq
    {
        output_high(Decoder0);
        output_low(Decoder1);
        output_low(Decoder2);
    }
    else if(dat == 7)    //0=low freq, 7=high freq
    {
        output_low(Decoder0);
        output_low(Decoder1);
        output_low(Decoder2);
    }
}

void Test()
{
    output_high(DecoderEnb);
    //PowerCheck();
    delay_ms(1);
    //while(1)
    //{
        output_high(LED_FM);
        output_low(LED_AM);
        output_high(P1A);
        output_low(Decoder0);
        output_low(Decoder1);
        output_low(Decoder2);
        delay_ms(300);
        output_low(P1A);
        output_high(Decoder0);
        output_low(Decoder1);
        output_low(Decoder2);
        delay_ms(300);
        output_low(Decoder0);
        output_high(Decoder1);
        output_low(Decoder2);
        delay_ms(300);
        output_high(Decoder0);
        output_high(Decoder1);
        output_low(Decoder2);
        delay_ms(300);
        output_low(LED_FM);
        output_high(LED_AM);
        output_low(Decoder0);
        output_low(Decoder1);
        output_high(Decoder2);
        delay_ms(300);
        output_high(Decoder0);
        output_low(Decoder1);
        output_high(Decoder2);
        delay_ms(300);
        output_low(Decoder0);
        output_high(Decoder1);
        output_high(Decoder2);
        delay_ms(300);
        output_high(Decoder0);
        output_high(Decoder1);
        output_high(Decoder2);
        delay_ms(300);
        output_low(DecoderEnb);
        delay_ms(300);
        output_high(DecoderEnb);
        
        //Test2();
    //}
}

void Test2()
{
    PowerCheck();
    lcd_init();
    output_low(LCD_LIGHT);
    output_high(PIN_A2);
    DSP_Init();
    Disp();
    while(1)
    {
        //DSP_Init();
        Disp();
        long freq1=795;
        float freq_flt;
        freq_flt=freq1;
        freq_flt=freq_flt/10;
        //printf(lcd_data,"FREQ= %3.1f MHz",freq_flt);
        //lcd_cmd(0xC0);  //0x80 1st line, 0xC0 2nd line
        //lcd_data("MYCOMKITS.com");
        //
        //unsigned int16 duty1=000;
        set_pwm5_duty(100);
        delay_ms(100);
        set_pwm5_duty(900);
        delay_ms(100);
        /*
        set_pwm5_duty(600);
        delay_ms(200);
        set_pwm5_duty(1000);

        Test();
        lcd_clear();
        delay_ms(300);
        */
    }

}

void Test3()
{
    
}
